use crate::cell::CellView;
use crate::transducer::{Publish, Publisher};
use core::cell::Cell;
#[cfg(feature = "embassy-sync")]
use embassy_sync::blocking_mutex::raw::RawMutex;
#[cfg(feature = "embassy-sync")]
use embassy_sync::pubsub::{Error, PubSubChannel, Subscriber};
pub use physical::transducer::input::*;
use physical::transducer::{State, Stateful};

#[cfg(feature = "embassy-sync")]
pub struct StatefulPublisher<
    T: Copy,
    MutexT: RawMutex,
    const CAPACITY: usize,
    const NUM_SUBS: usize,
> {
    pub state: State<T>,
    pub publisher: Publisher<T, MutexT, CAPACITY, NUM_SUBS>,
}

impl<T: Copy, MutexT: RawMutex, const CAPACITY: usize, const NUM_SUBS: usize>
    StatefulPublisher<T, MutexT, CAPACITY, NUM_SUBS>
{
    #[inline(always)]
    pub fn new(
        state: State<T>,
        publisher: Publisher<T, MutexT, CAPACITY, NUM_SUBS>,
    ) -> Self {
        Self { state, publisher }
    }

    #[inline(always)]
    pub fn update(&self, value: T) {
        self.state.update(value);
        self.publisher.update(value);
    }
}

impl<T: Copy, MutexT: RawMutex, const CAPACITY: usize, const NUM_SUBS: usize> Stateful
    for StatefulPublisher<T, MutexT, CAPACITY, NUM_SUBS>
{
    type Value = T;

    #[inline(always)]
    fn state_cell(&self) -> CellView<Self::Value> {
        self.state.state_cell()
    }

    #[inline(always)]
    fn state(&self) -> Self::Value {
        self.state.state()
    }
}

impl<T: Copy, MutexT: RawMutex, const CAPACITY: usize, const NUM_SUBS: usize>
    Publish<CAPACITY, NUM_SUBS> for StatefulPublisher<T, MutexT, CAPACITY, NUM_SUBS>
{
    type Value = T;
    type Mutex = MutexT;

    #[inline(always)]
    fn subscribe(
        &self,
    ) -> Result<Subscriber<Self::Mutex, Self::Value, CAPACITY, NUM_SUBS, 0>, Error> {
        self.publisher.subscribe()
    }
}
